# EKS node pool setup

resource "aws_iam_role" "node" {
  name = "${var.cluster_name}-node"

  assume_role_policy = jsonencode({
    Statement = [{
      Action = "sts:AssumeRole"
      Effect = "Allow"
      Principal = {
        Service = "ec2.amazonaws.com"
      }
    }]
    Version = "2012-10-17"
  })
}

resource "aws_iam_role_policy_attachment" "nodes_AmazonEKSWorkerNodePolicy" {
  policy_arn = "arn:aws:iam::aws:policy/AmazonEKSWorkerNodePolicy"
  role       = aws_iam_role.node.name
}

resource "aws_iam_role_policy_attachment" "nodes_AmazonEKS_CNI_Policy" {
  policy_arn = "arn:aws:iam::aws:policy/AmazonEKS_CNI_Policy"
  role       = aws_iam_role.node.name
}

resource "aws_iam_role_policy_attachment" "nodes_AmazonEC2ContainerRegistryReadOnly" {
  policy_arn = "arn:aws:iam::aws:policy/AmazonEC2ContainerRegistryReadOnly"
  role       = aws_iam_role.node.name
}

# prepare the policy for autoscaling if needed
resource "aws_iam_role_policy_attachment" "autoscaler" {
  role       = aws_iam_role.node.name
  policy_arn = aws_iam_policy.cluster_autoscaler.arn
}

resource "aws_iam_policy" "cluster_autoscaler" {
  name_prefix = "cluster-autoscaler"
  description = "EKS cluster-autoscaler policy for cluster ${var.cluster_name}"
  policy      = data.aws_iam_policy_document.cluster_autoscaler.json
}

data "aws_iam_policy_document" "cluster_autoscaler" {
  statement {
    sid    = "clusterAutoscalerAll"
    effect = "Allow"

    actions = [
      "autoscaling:DescribeAutoScalingGroups",
      "autoscaling:DescribeAutoScalingInstances",
      "autoscaling:DescribeLaunchConfigurations",
      "autoscaling:DescribeTags",
      "ec2:DescribeLaunchTemplateVersions",
      "autoscaling:SetDesiredCapacity",
      "autoscaling:TerminateInstanceInAutoScalingGroup",
      "autoscaling:UpdateAutoScalingGroup",
    ]

    resources = ["*"]
  }
}

locals {
  node_pools_expanded = { for k, v in var.node_pools : v["name"] => merge(
    {
      key_name           = ""
      instance_type      = "t3.medium"
      instance_count_max = 3
      instance_count_min = 1
      instance_disk_size = 20
      labels             = {}
      tags               = {}
    },
    v)
  }
}

# create a node pool
resource "aws_eks_node_group" "nodes" {
  for_each = local.node_pools_expanded

  cluster_name = aws_eks_cluster.cluster.name
  version      = var.k8s_version

  node_role_arn = aws_iam_role.node.arn
  subnet_ids    = var.nodes_subnet_ids

  node_group_name = each.value["name"]
  instance_types  = [each.value["instance_type"]]
  disk_size       = each.value["instance_disk_size"]
  labels          = each.value["labels"]

  scaling_config {
    desired_size = each.value["instance_count_min"]
    max_size     = each.value["instance_count_max"]
    min_size     = each.value["instance_count_min"]
  }

  dynamic "remote_access" {
    for_each = each.value["key_name"] != "" ? [{
      ec2_ssh_key               = each.value["key_name"]
      source_security_group_ids = lookup(each.value, "source_security_group_ids", [])
    }] : []

    content {
      ec2_ssh_key               = remote_access.value["ec2_ssh_key"]
      source_security_group_ids = remote_access.value["source_security_group_ids"]
    }
  }

  # Ensure that IAM Role permissions are created before and deleted after EKS Node Group handling.
  # Otherwise, EKS will not be able to properly delete EC2 Instances and Elastic Network Interfaces.
  depends_on = [
    aws_iam_role_policy_attachment.nodes_AmazonEKSWorkerNodePolicy,
    aws_iam_role_policy_attachment.nodes_AmazonEKS_CNI_Policy,
    aws_iam_role_policy_attachment.nodes_AmazonEC2ContainerRegistryReadOnly,
    kubernetes_config_map.aws_auth
  ]

  lifecycle {
    ignore_changes = [scaling_config[0].desired_size]
  }

  tags = each.value["tags"]
}
